"""
MTR protocol parser module

Classes
    TransmitterType

Functions
    mtr_response_to_json(payload, transmitters_metadata)
"""

from enum import Enum
import json
import logging
from datetime import datetime, timedelta, timezone
from collections import namedtuple
from collections import defaultdict
from mtr2mqtt import metadata

class TransmitterType(Enum):
    """ MTR series Transmitter types """
    FT10 = 0
    MTR262 = 2
    MTR264 = 3
    MTR265 = 5
    MTR165 = 6
    FTR860 = 7
    CSR264START = 8
    CSR264LIMIT = 9
    CSR260 = 11
    KMR260 = 12
    UTILITY = 15


def check_payload(payload):
    """
    Verifies that payload length matches lenght field
    and that the packet is long enough
    """

    if payload is None:
        logging.warning("No MTR respose to parse")
        return False
    if len(payload) > 4:
        payload_fields = str(payload).split(" ")
        data_bytes = int(payload_fields[1]) >> 5
        # Actual payload bytes = received - type - length&voltage - rsl - id
        received_data_bytes = len(payload_fields)-4
        if data_bytes == received_data_bytes:
            return True
        logging.warning(
            "Payload size mismatch, expected %s got %s bytes",
            data_bytes, received_data_bytes
            )
    elif len(payload) <= 4:
        logging.warning("Too short payload to process: %s", payload)
    return False


def _get_header_fields(payload):
    """
    Get transmitter type, rsl, id and voltage fields
    """
    if check_payload(payload):
        payload_fields = str(payload).split(" ")
        transmitter_type = TransmitterType(int(payload_fields[0])).name
        battery_voltage = (
            int(payload_fields[1]) & 31
        ) / 10  # clear 3 highest bits used for data length (31 = 00011111)
        transmitter_id = str(payload_fields[3])
        rsl = (int(payload_fields[2]) & 127) - 127
        Headers = namedtuple('Headers', 'transmitter_type rsl transmitter_id battery_voltage')
        return Headers(transmitter_type, rsl, transmitter_id, battery_voltage)
    return None

def _get_ft10_reading(payload):
    """
    Get FT10/MTR260/CSR260 reading from payload
    """
    if check_payload(payload):
        payload_fields = str(payload).split(" ")
        reading = round(
            (int(payload_fields[4]) + int(payload_fields[5]) * 256) / 10.0 - 273.2,
            1,
        )
        return reading
    return None

def _get_ft10_data(headers, payload):
    """
    Get FT10/MTR260/CSR260 reading as json
    """
    return {
        "battery": headers.battery_voltage,
        "type": f"{headers.transmitter_type}",
        "rsl": headers.rsl,
        "id": headers.transmitter_id,
        "reading": _get_ft10_reading(payload),
        "timestamp": f"{datetime.now(timezone.utc)}",
    }

def _get_utility_data(headers, payload):
    """
    Get Utility packet as json
    """
    payload_fields = str(payload).split(" ")
    # Some transmitters may intermittently send some additional
    # information using transmitter type 15.
    # After that, comes a byte indicating the type of utility data,
    # and after that, some data.
    # Currently only 0 and 1 types seem to exist
    logging.debug("Utility packet MTR payload: %s", payload)
    # Calibration date
    if payload_fields[4] == "0" and len(payload_fields) == 7:
        # Rest of the payload is a 16 bit word, least significant byte first.
        # Value 0 corresponds to 1.1.2000, and every day increments by one.
        start_date = datetime.strptime("1.1.2000", "%d.%m.%Y")
        logging.debug("Utility packet, payload length: %s", len(payload_fields))
        calibration_days = int(payload_fields[5]) + 256 * int(payload_fields[6])
        if (
            calibration_days == 65535
        ):
            # Guessing this is a special value indicating that the transmitter
            # is not calibrated
            logging.debug(
                "Calibration utility packet with no calibration information, skipping"
            )
            return {
                "battery": headers.battery_voltage,
                "type": f"{headers.transmitter_type}",
                "rsl": headers.rsl,
                "id": headers.transmitter_id,
                "message": "Device not calibrated" ,
                "timestamp": f"{datetime.now(timezone.utc)}"
                }
        calibration_date = start_date + timedelta(days=calibration_days)
        return {
            "battery": headers.battery_voltage,
            "type": f"{headers.transmitter_type}",
            "rsl": headers.rsl,
            "id": headers.transmitter_id,
            "calibrated": f"{calibration_date.strftime('%d.%m.%Y')}",
            "timestamp": f"{datetime.now(timezone.utc)}"
            }
    # Find out the specs for Utility packet with type 1 as identifier
    # Utility packet MTR payload: 15 252 47 27054 1 1 23 80 161 160 170
    # Utility packet MTR payload: 15 252 55 28506 1 1 23 80 162 134 208
    # Utility packet MTR payload: 15 252 42 23511 1 11 12 80 159 217 215
    return {
        "battery": headers.battery_voltage,
        "type": f"{headers.transmitter_type}",
        "rsl": headers.rsl,
        "id": headers.transmitter_id,
        "utility": "Unknown utility packet",
        "timestamp": f"{datetime.now(timezone.utc)}"
        }

def _default_payload_handler():
    """
    Wrapper function for _unsupported_data_type to be used while processing payloads
    """

    def _unsupported_data_type(headers, payload):
        """
        Get unsupported package response as json
        """
        logging.debug("Unsupported response, payload: %s", payload)
        return {
            "battery": headers.battery_voltage,
            "type": f"{headers.transmitter_type}",
            "rsl": headers.rsl,
            "id": headers.transmitter_id,
            "message": "Unsupport transmitter type" ,
            "timestamp": f"{datetime.now(timezone.utc)}"
            }
    return _unsupported_data_type


payload_type_function_mapping = {
    "FT10" : _get_ft10_data,
    "CSR260" :_get_ft10_data,
    "UTILITY" : _get_utility_data
    }

payload_proressor = defaultdict(_default_payload_handler, payload_type_function_mapping)

def mtr_response_to_json(payload, transmitters_metadata):
    """
    Convert MTR response packet to JSON

    Optionally adding metadata from a file
    """

    headers = _get_header_fields(payload)

    if headers:

        data = payload_proressor[headers.transmitter_type](headers, payload)

        if transmitters_metadata:
            transmitter_information = metadata.get_data(
                headers.transmitter_id, transmitters_metadata
            )
            logging.debug("Transmitter info: %s", transmitter_information)
            if transmitter_information:
                data_with_transmitter_info = {**data, **transmitter_information}
                return json.dumps(data_with_transmitter_info)
        return json.dumps(data)


    return None



"""
Module for getting metadata for transmitters

Functions
    loadfile(file)
    get_data(transmitter_id, all_transmitters)

"""

import logging
import sys
import json
import yaml

def loadfile(file):
    """
    Loads metadata file and returns it as json string
    """
    try:
        with open(file,'r') as metafile:
            transmitter_info = yaml.safe_load(metafile)
            return json.dumps(transmitter_info)

    except FileNotFoundError:
        logging.exception("File not found")
        sys.exit(-1)

def get_data(transmitter_id, all_transmitters):
    """
    Gets metada for transmitter
    """
    all_transmitters = json.loads(all_transmitters)
    logging.debug("All transmitters metadata: %s", all_transmitters)
    logging.debug("Transmitter id: %s", transmitter_id)
    info = None
    for transmitter in all_transmitters:
        logging.debug("Iterated transmitter: %s", transmitter)
        if transmitter['id'] == int(transmitter_id):
            info = transmitter.copy()
            info.pop('id', None)
            break
    logging.debug("Transmitter: %s metadata: %s", transmitter_id, info)
    return info



"""
Command line module


Functions
    create_parser()
    main()

"""
from argparse import ArgumentParser
import logging
import sys
import time
import os
import paho.mqtt.client as mqtt
from serial.tools import list_ports
from serial.serialutil import EIGHTBITS, FIVEBITS, SEVENBITS, SIXBITS
import serial
from mtr2mqtt import scl
from mtr2mqtt import mtr
from mtr2mqtt import metadata


def create_parser():
    """
    Cli module argument parser
    """
    parser = ArgumentParser(description="""
    MTR receiver readings to MQTT topic as json. 
    Most options can be also configured using environment variables.
    """)

    parser.add_argument(
        "--serial-port", '-s',
        help="Serial port for MTR series receiver (ENV: MTR2MQTT_SERIAL_PORT)",
        default=os.environ.get('MTR2MQTT_SERIAL_PORT'),
        required=False
        )
    parser.add_argument(
        "--baudrate",
        help="Serial port baud rate",
        default=int(os.environ.get('MTR2MQTT_BAUDRATE', 9600)),
        required=False,
        type=int,
        choices=[9600, 115200]
        )
    parser.add_argument(
        "--bytesize",
        help="Serial port byte size",
        default=EIGHTBITS,
        required=False,
        choices=[FIVEBITS, SIXBITS,SEVENBITS, EIGHTBITS]
        )
    parser.add_argument(
        "--parity", help="Serial port parity",
        default=serial.PARITY_NONE,
        required=False,
        choices=[
            serial.PARITY_NONE,
            serial.PARITY_EVEN,
            serial.PARITY_ODD,
            serial.PARITY_MARK,
            serial.PARITY_SPACE
            ]
        )
    parser.add_argument(
        "--stopbits",
        help="Serial port stop bits",
        default=serial.STOPBITS_ONE,
        required=False,
        choices=[serial.STOPBITS_ONE, serial.STOPBITS_TWO]
        )
    parser.add_argument(
        "--serial-timeout",
        help="Timeout for serial port (ENV: MTR2MQTT_SERIAL_TIMEOUT)",
        default=int(os.environ.get('MTR2MQTT_SERIAL_TIMEOUT', 1)),
        required=False
        )
    parser.add_argument(
        "--scl-address",
        help="SCL address 0...123 or 126 for broadcast (ENV: MTR2MQTT_SCL_ADDRESS)",
        default=int(os.environ.get('MTR2MQTT_SCL_ADDRESS', 126)),
        required=False,
        type=int,
        choices=(list(range(124))+[126]),
        metavar='' # for hiding the messy choices output
        )
    parser.add_argument(
        "--mqtt-host", '-m',
        help="MQTT host address (ENV: MTR2MQTT_MQTT_HOST)",
        default=os.environ.get('MTR2MQTT_MQTT_HOST'),
        required=False)
    parser.add_argument(
        "--mqtt-port", '-p',
        help="MQTT host port (ENV: MTR2MQTT_MQTT_PORT)",
        default=int(os.environ.get('MTR2MQTT_MQTT_PORT', 1883)),
        required=False)
    parser.add_argument(
        "--metadata-file",'-f',
        help="A file for transmitter metadata (ENV: MTR2MQTT_METADATA_FILE)",
        default=os.environ.get('MTR2MQTT_METADATA_FILE'),
        required=False,
        type=str
        )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "--debug", '-d',
        help="Enable pringing debug messages (ENV: MTR2MQTT_DEBUG)",
        required=False,
        default=os.environ.get('MTR2MQTT_DEBUG', 'False').lower() in ['true', '1'],
        action='store_true'
        )
    group.add_argument(
        "--quiet", '-q',
        help="Print only error messages (ENV: MTR2MQTT_QUIET)",
        required=False,
        default=os.environ.get('MTR2MQTT_QUIET', 'False').lower() in ['true', '1'],
        action='store_true'
        )

    return parser


def _open_receiver_port(args):
    """
    Open serial port connection
    """

    # Trying to find MTR compatible receiver
    # Filtering ports with Nokeval manufactured models that MTR receivers might use
    serial_ports = list(list_ports.grep('RTR|FTR|DCS'))

    ser = serial.Serial()

    # Check if serial port was given as argument
    if args.serial_port:
        try:
            ser = serial.Serial(
                port=args.serial_port,
                baudrate=args.baudrate,
                bytesize=args.bytesize,
                parity=args.parity,
                stopbits=args.stopbits,
                timeout=args.serial_timeout
                )
            if ser.is_open:
                device_type = scl.get_receiver_type(ser,args.scl_address)
                if device_type:
                    print(f"Connected device type: {device_type}")
        except serial.serialutil.SerialException:
            logging.exception("Unable to open serial port")
            sys.exit(-1)
        except ValueError:
            logging.exception("Unable to open serial port")
            sys.exit(-1)
    else:
        for port in serial_ports:
            try:
                ser = serial.Serial(
                    port=port.device,
                    baudrate=args.baudrate,
                    bytesize=args.bytesize,
                    parity=args.parity,
                    stopbits=args.stopbits,
                    timeout=args.serial_timeout
                    )
                if ser.is_open:
                    device_type = scl.get_receiver_type(ser,args.scl_address)
                    if device_type:
                        print(f"Connected device type: {device_type}")
            except serial.serialutil.SerialException:
                pass
    return ser

def _get_latest_from_ring_buffer(ser, scl_dbg_1_command, serial_config):
    """"
    Get latest packet from MTR receiver ring buffer
    """
    try:
        ser.write(scl_dbg_1_command)
        logging.debug("Wrote message: %s to: to %s", scl_dbg_1_command, ser.name)
        response = ser.read_until(scl.END_CHAR)
        response_checksum = bytes(ser.read(1))
        parsed_response = scl.parse_response(response,response_checksum)
        logging.debug("response: %s, response checksum: %s", response, response_checksum)
        logging.debug("parsed SCL response: %s", parsed_response)
    except OSError:
        logging.exception("OS Error: Reading to serial port failed")
        ser.close()
        time.sleep(1)
        try:
            logging.warning("Trying to reopen serial port")
            ser.apply_settings(serial_config)
            ser.open()
        except serial.serialutil.SerialException:
            logging.exception("Serial exception: opening serial port failed")
            time.sleep(1)
        except FileNotFoundError:
            logging.exception("File not found: opening serial port failed")
            time.sleep(1)
        except OSError:
            logging.exception("OS Error: opening serial port failed")
            time.sleep(1)
    return parsed_response

def _open_mqtt_connection(args):
    """
    Get MQTT client
    """
    client = mqtt.Client('mtr2mqtt')
    mqtt_host = 'localhost'
    mqtt_port = 1883
    if args.mqtt_host:
        mqtt_host = args.mqtt_host

    if args.mqtt_port:
        mqtt_port = int(args.mqtt_port)

    print(f"Connecting to MQTT host: {mqtt_host}:{mqtt_port}")
    try:
        client.connect(mqtt_host,mqtt_port)
    except mqtt.socket.timeout:
        logging.exception("Unable to connect to MQTT host")
        sys.exit(-1)
    return client

def main():
    """
    Main function
    """

    args = create_parser().parse_args()

    # Configure logging
    log_format ='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    if args.debug:
        logging.basicConfig(level=logging.DEBUG, format=log_format)
        print("Debug logging enabled")
    elif args.quiet:
        logging.basicConfig(level=logging.WARNING)
    else:
        logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(message)s')

    # meta data file processing
    if args.metadata_file:
        transmitters_metadata = metadata.loadfile(args.metadata_file)

    else:
        transmitters_metadata = None

    # SCL constants
    scl_sn_command = scl.create_command('SN ?',args.scl_address)
    scl_dbg_1_command = scl.create_command('DBG 1 ?',args.scl_address)

    ser = _open_receiver_port(args)

    if not ser.is_open:
        logging.fatal("Unable to find MTR receivers")
        sys.exit(-1)

    serial_config = ser.get_settings()

    ser.write(scl_sn_command)
    logging.debug("Wrote message: %s to: %s", scl_sn_command, ser.name)
    response = ser.read_until(scl.END_CHAR)
    response_checksum = bytes(ser.read(1))
    print(f"Receiver S/N: {scl.parse_response(response,response_checksum)}")

    mqtt_client = _open_mqtt_connection(args)

    while True:

        response = _get_latest_from_ring_buffer(ser, scl_dbg_1_command, serial_config)
        # Character # is returned when the buffer is empty
        if response != "#":
            measurement_json = mtr.mtr_response_to_json(response, transmitters_metadata)
            if measurement_json:
                logging.info(measurement_json)
                mqtt_client.publish('measurements', payload=measurement_json, qos=0, retain=False)
        else:
            logging.debug('Ring buffer empty')
            time.sleep(1)

if __name__ == "__main__":
    main()



"""
Tests for MTR module
"""
import pytest

from context import mtr2mqtt
from mtr2mqtt import scl

SCL_COMMAND_MEA_CH_1_OUTPUT = b'\x81\x4D\x45\x41\x20\x43\x48\x20\x31\x20\x3F\x03\x6F'
SCL_COMMAND_TYPE_OUTPUT = b'\x80TYPE ?\x03\x04'

def test_scl_bcc_with_correct_checksum():
    """
    scl.calc_bcc returns the checksum byte
    """
    assert scl.calc_bcc(b'\x060 91 56 24859 169 11\x03') == b'\x12'

def test_scl_command_creation_without_address():
    """
    scl.create_command returns the SCL command in bytes using default broadcast address
    """
    assert scl.create_command('TYPE ?') == b'\xfeTYPE ?\x03\x04'

def test_scl_command_creation_with_address():
    """
    scl.create_command returns the SCL command in bytes using defined address
    """
    assert scl.create_command('TYPE ?',0) == SCL_COMMAND_TYPE_OUTPUT

def test_scl_mea_ch_command_creation_with_address():
    """
    scl.create_command returns the SCL command in bytes using defined address
    """
    assert scl.create_command('MEA CH 1 ?',1) == SCL_COMMAND_MEA_CH_1_OUTPUT



"""
Tests for metadata module
"""
import json
from context import mtr2mqtt
from mtr2mqtt import metadata


METADATA_TEST_FILE = "tests/metadata.yml"
METADATA_TEST_FILE_OUTPUT = json.dumps(
    [
        {
            'id': 1234,
            'location': 'Room',
            'unit': 'C'
        },
        {
            'id': 2345,
            'location': 'Room 2',
            'unit': '%'
        }
    ])
METADATA_TRANSMITTER_ID = 2345

METADATA_TEST_TRANSMITTER_OUTPUT = {'location': 'Room 2', 'unit': '%'}

def test_metadata_file_load():
    """
    metadata.loadfile returns transmitter metadata as json
    """
    assert metadata.loadfile(METADATA_TEST_FILE) == METADATA_TEST_FILE_OUTPUT

def test_metadata_get_data():
    """
    metadata.get_data returns transmitter metadata
    """
    assert metadata.get_data(
        METADATA_TRANSMITTER_ID,
        METADATA_TEST_FILE_OUTPUT) ==  METADATA_TEST_TRANSMITTER_OUTPUT



"""
SCL protocol parser module

Functions
    calc_bcc(message)
    create_command(scl_command, scl_address=126)
    parse_response(scl_response, scl_response_checksum)
    get_receiver_type(ser, scl_address)

"""

import logging
import serial


# Constants
END_CHAR = b"\x03"


def calc_bcc(message):
    """
    Calculates SCL packet checksum using XOR over bytes
    """
    bcc = 0
    for item in message:
        bcc = bcc ^ item
    return bytes([bcc])


def create_command(scl_command, scl_address=126):
    """
    SCL command packet format is:
    <ID><commmand><EXT><BCC>
    ID is address 0...123 + 128
    EXT is end character byte having value 3
    BCC is checksum byte that is calculated using XOR operation of command and ext bytes
    SCL address 126 is broadcast address
    """
    ext = chr(3)
    scl_id = bytes([scl_address + 128])  # ID is formed by adding 128 to address
    bcc = calc_bcc(str.encode(scl_command + ext))
    return scl_id + str.encode(scl_command + ext) + bcc


def parse_response(scl_response, scl_response_checksum):
    """
    Checks SCL response checksum and returns payload as string
    """
    # Check that checksum matches
    scl_tmp_response_bytes = [scl_response[i : i + 1] for i in range(len(scl_response))]
    calulated_response_checksum = calc_bcc(scl_response)
    if calulated_response_checksum != scl_response_checksum:
        logging.warning('Checsum error, ignoring response')
        logging.debug(
            "SCL checksum error, response: %s, received checksum: %s expected checksum: %s",
            scl_response, scl_response_checksum, calulated_response_checksum
        )
        return None
    try:
        scl_response_content = b"".join(
            scl_tmp_response_bytes[1 : len(scl_tmp_response_bytes) - 1]
        ).decode("ascii")
    except UnicodeDecodeError:
        logging.warning("Unable to parse SCL response: %s", scl_response)
        return None
    logging.debug("Parsed response: %s", scl_response_content)
    return scl_response_content



def _parse_receiver(ser):
    """
    Parse valid receiver from SCL response
    """
    supported_device_types = ["MTR970", "RTR970", "FTR980", "CSR970"]
    response = ser.read_until(END_CHAR)
    if response:
        response_checksum = bytes(ser.read(1))
        expected_checksum = calc_bcc(response)
        if response_checksum == expected_checksum:
            parsed_response = parse_response(response, response_checksum)
            valid_device_type = any(
                substring in parsed_response
                for substring in supported_device_types
            )
            if valid_device_type:
                return parsed_response
            logging.debug(
                "Received incorrect device type response: %s", parsed_response
            )
        logging.debug("Incorrect checksum received")
    return None


def get_receiver_type(ser, scl_address):
    """
    Returns MTR receiver type
    """
    test_command = create_command("TYPE ?", scl_address)
    logging.info("Checking device type in port: %s", ser.name)
    try:
        ser.reset_input_buffer()
        ser.reset_output_buffer()
        logging.debug("waiting bytes in input buffer after reset: %s", ser.in_waiting)
        ser.write(test_command)
        logging.debug(
            "Wrote message: %s to: %s using settings: %s",
            test_command, ser.name, ser.get_settings()
        )
        receiver = _parse_receiver(ser)
        if receiver:
            return receiver
        # Because of FTDI USB to Serial port converter used in receivers might buffer
        # previous full or partial unread response despite resetting buffer, lets try to check
        # next answer
        receiver = _parse_receiver(ser)
        if receiver:
            return receiver
        ser.close()
        return None
    except serial.serialutil.SerialException:
        return None



"""
Tests for cli module
"""
import pytest
from context import mtr2mqtt
from mtr2mqtt import cli

MTR2MQTT_ARGS = "--mqtt localhost"

# def test_parser_without_mqtt_server():
#     """
#     Without a specified mqtt server the parser will exit.
#     """
#     with pytest.raises(SystemExit):
#         parser = cli.create_parser()
#         parser.parse_args([])

def test_parser_with_mqtt_server_option_without_value():
    """
    Without a specified mqtt server the parser will exit.
    """
    with pytest.raises(SystemExit):
        parser = cli.create_parser()
        parser.parse_args(["--mqtt-host"])

def test_parser_with_mqtt_server():
    """
    The parser will not exit if it receives mqtt option and value
    """
    parser = cli.create_parser()
    args = parser.parse_args(["--mqtt-host", "localhost"])

    assert args.mqtt_host == 'localhost'



from setuptools import setup, find_packages

with open('README.rst', encoding='UTF-8') as f:
    readme = f.read()

__version__ = "0.3.2"

setup(
    name='mtr2mqtt',
    version=__version__,
    description='MTR receiver readings to MQTT server',
    author='topo',
    author_email='tvallas@iki.fi',
    url='https://github.com/tvallas/mtr2mqtt',
    install_requires=['pyserial', 'paho-mqtt', 'PyYAML'],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'mtr2mqtt=mtr2mqtt.cli:main',
        ]
    }
)


"""
Tests for MTR module
"""
#import pytest

import json
from freezegun import freeze_time
from mtr2mqtt import mtr


MTR_READING_INPUT = '0 90 58 15006 145 11'
MTR_READING_OUTPUT = json.dumps({
    "battery": 2.6,
    "type": "FT10",
    "rsl": -69,
    "id": "15006",
    "reading": 22.9,
    "timestamp": "2020-09-23 19:34:13.497019+00:00"})

MTR_UNSUPPORTED_INPUT = '2 90 58 12345 145 11'
MTR_UNSUPPORTED_OUTPUT = json.dumps({
    "battery": 2.6,
    "type": "MTR262",
    "rsl": -69,
    "id": "12345",
    "message": "Unsupport transmitter type",
    "timestamp": "2020-09-23 19:34:13.497019+00:00"})

MTR_CALIBRATION_INPUT ='15 124 45 27054 0 184 23'
MTR_CALIBRATION_OUTPUT = json.dumps({
    "battery": 2.8,
    "type": "UTILITY",
    "rsl": -82,
    "id": "27054",
    "calibrated": "16.08.2016",
    "timestamp": "2020-09-23 19:34:13.497019+00:00"})

MTR_NONE_MESSAGE = None
MTR_TOO_SHORT_INPUT = '0 90 58 15006'

@freeze_time("2020-09-23 19:34:13.497019+00:00")
def test_mtr_reading_response_to_json_with_sample_input():
    """
    mtr.mtr_response_to_json returns reading message formatted as json
    """
    assert mtr.mtr_response_to_json(MTR_READING_INPUT, None) == MTR_READING_OUTPUT

@freeze_time("2020-09-23 19:34:13.497019+00:00")
def test_mtr_reading_response_to_json_with_unsupported_input():
    """
    mtr.mtr_response_to_json returns error message formatted as json
    """
    assert mtr.mtr_response_to_json(MTR_UNSUPPORTED_INPUT, None) == MTR_UNSUPPORTED_OUTPUT

@freeze_time("2020-09-23 19:34:13.497019+00:00")
def test_mtr_calibdation_response_to_json_with_sample_input():
    """
    mtr.mtr_response_to_json calibration message formatted as json
    """
    assert mtr.mtr_response_to_json(MTR_CALIBRATION_INPUT, None) == MTR_CALIBRATION_OUTPUT

def test_check_payload_with_valid_message():
    """
    mtr.check_payload checks the validity of payload and returns boolean
    """
    assert mtr.check_payload(MTR_READING_INPUT)

def test_check_payload_with_no_message():
    """
    mtr.check_payload checks the validity of payload and returns boolean
    """
    assert not mtr.check_payload(MTR_NONE_MESSAGE)

def test_check_payload_with_short_message():
    """
    mtr.check_payload checks the validity of payload and returns boolean
    """
    assert not mtr.check_payload(MTR_TOO_SHORT_INPUT)

def test_get_headers_with_valid_message():
    """
    mtr.get_headers returns header fields as tuple
    """
    assert mtr._get_header_fields(MTR_READING_INPUT) == ('FT10', -69, '15006', 2.6)



"""
MQTT module
"""

import logging
import paho.mqtt.client as mqtt

client = mqtt.Client()

def connect(host, port):
    """
    Handles mqtt connection
    """
    client.enable_logger()
    client.on_connect = _on_connect
    client.on_publish = _on_publish
    return client.connect(host=host,port=port)


def _on_connect():
    """
    On connect handler
    """

    logging.info("Connected to MQTT host")

def _on_publish(topic, payload):
    """
    On publish handler
    """
    logging.info("Connected to MQTT host")
    client.publish(topic=topic, payload=payload)
